# Generated by Django 5.2.11 on 2026-02-27 15:05

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('system', '0003_alter_apps_apps_type_alter_basetask_task_type_and_more'),
    ]

    operations = [
        migrations.AlterModelTable(
            name='aichatmessage',
            table='base_ai_ai_chat_message',
        ),
        migrations.AlterModelTable(
            name='aichatsession',
            table='base_ai_ai_chat_session',
        ),
        migrations.AlterModelTable(
            name='aideskappmanage',
            table='base_ai_ai_desk_app_manage',
        ),
        migrations.AlterModelTable(
            name='apiwhitelist',
            table='base_ai_api_white_list',
        ),
        migrations.AlterModelTable(
            name='apps',
            table='base_ai_apps',
        ),
        migrations.AlterModelTable(
            name='area',
            table='base_ai_system_area',
        ),
        migrations.AlterModelTable(
            name='basetask',
            table='base_ai_base_task',
        ),
        migrations.AlterModelTable(
            name='chunkuploadsession',
            table='base_ai_system_chunk_upload_session',
        ),
        migrations.AlterModelTable(
            name='collaborator',
            table='base_ai_collaborator',
        ),
        migrations.AlterModelTable(
            name='corebaseauth',
            table='base_ai_auth',
        ),
        migrations.AlterModelTable(
            name='dept',
            table='base_ai_system_dept',
        ),
        migrations.AlterModelTable(
            name='dictionary',
            table='base_ai_system_dictionary',
        ),
        migrations.AlterModelTable(
            name='externalrely',
            table='base_ai_external_rely',
        ),
        migrations.AlterModelTable(
            name='filelist',
            table='base_ai_system_file_list',
        ),
        migrations.AlterModelTable(
            name='folderlist',
            table='base_ai_system_folder_list',
        ),
        migrations.AlterModelTable(
            name='frequentlyusedcontacts',
            table='base_ai_frequently_used_contacts',
        ),
        migrations.AlterModelTable(
            name='imchatqa',
            table='base_ai_im_chat_qa',
        ),
        migrations.AlterModelTable(
            name='imchatquestion',
            table='base_ai_im_chat_question',
        ),
        migrations.AlterModelTable(
            name='imchatquestiongroup',
            table='base_ai_im_chat_question_group',
        ),
        migrations.AlterModelTable(
            name='imchatrecordform',
            table='base_ai_im_chat_record_form',
        ),
        migrations.AlterModelTable(
            name='imchattogroup',
            table='base_ai_im_chat_to_group',
        ),
        migrations.AlterModelTable(
            name='imchattogroupmessage',
            table='base_ai_im_chat_to_group_message',
        ),
        migrations.AlterModelTable(
            name='imchattouser',
            table='base_ai_im_chat_to_user',
        ),
        migrations.AlterModelTable(
            name='imsession',
            table='base_ai_im_session',
        ),
        migrations.AlterModelTable(
            name='loginlog',
            table='base_ai_system_login_log',
        ),
        migrations.AlterModelTable(
            name='menu',
            table='base_ai_system_menu',
        ),
        migrations.AlterModelTable(
            name='menubutton',
            table='base_ai_system_menu_button',
        ),
        migrations.AlterModelTable(
            name='messagecenter',
            table='base_ai_message_center',
        ),
        migrations.AlterModelTable(
            name='messagecentertargetuser',
            table='base_ai_message_center_target_user',
        ),
        migrations.AlterModelTable(
            name='mysuperiors',
            table='base_ai_my_superiors',
        ),
        migrations.AlterModelTable(
            name='operationlog',
            table='base_ai_system_operation_log',
        ),
        migrations.AlterModelTable(
            name='post',
            table='base_ai_system_post',
        ),
        migrations.AlterModelTable(
            name='role',
            table='base_ai_system_role',
        ),
        migrations.AlterModelTable(
            name='safetycheckrecord',
            table='base_ai_safety_check_record',
        ),
        migrations.AlterModelTable(
            name='sendmessagetask',
            table='base_ai_send_message_task',
        ),
        migrations.AlterModelTable(
            name='systemconfig',
            table='base_ai_system_config',
        ),
        migrations.AlterModelTable(
            name='systemusage',
            table='base_ai_system_usage',
        ),
        migrations.AlterModelTable(
            name='tags',
            table='base_ai_tags',
        ),
        migrations.AlterModelTable(
            name='taskrunhistory',
            table='base_ai_task_run_history',
        ),
        migrations.AlterModelTable(
            name='users',
            table='base_ai_system_users',
        ),
    ]
